package com.hisschemoller.model 
{
	import org.puremvc.as3.interfaces.IProxy;
	import org.puremvc.as3.patterns.proxy.Proxy;
	
	import flash.events.SampleDataEvent;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.utils.getTimer;
	import flash.utils.setTimeout;	
	/**
	 * @author Wouter Hisschemöller
	 * (c) Jul 12, 2009
	 */
	public class AudioDriverProxy extends Proxy implements IProxy 
	{
		public static const NAME : String = "AudioDriverProxy";
		public static const BUFFER_SIZE : int = 3072;
		public static const SAMPLE_RATE : int = 44100;
		public static const BYTES_PER_SAMPLE : int = 8;
		private const BUFFER : Vector.<Vector.<Number>> = new Vector.<Vector.<Number>>(2, true);
		private const SOUND : Sound = new Sound();
		private var _soundChannel : SoundChannel;
		private var _soundLatency : Number = 0;
		private var _lastBuffer : int;
		private var _engine : AudioEngineProxy;

		public function AudioDriverProxy(data : Object = null)
		{
			super(NAME, data);
			
			// Stereo (2 channels)
			BUFFER[0] = new Vector.<Number>(BUFFER_SIZE, true);
			BUFFER[1] = new Vector.<Number>(BUFFER_SIZE, true);
			
			SOUND.addEventListener(SampleDataEvent.SAMPLE_DATA, handleSampleData);
		}
		
		/**
		 * 
		 */
		override public function onRegister() : void
		{
			_engine = facade.retrieveProxy(AudioEngineProxy.NAME) as AudioEngineProxy;
			
			// delay 500ms to avoid click noise at startup
			setTimeout(start, 500);
		}

		/**
		 * @return : latency in milliSeconds
		 */
		public function getLatency() : Number
		{
			return _soundLatency;
		}

		/**
		 * 
		 */
		private function start() : void
		{
			_lastBuffer = getTimer();
			
			_soundChannel = SOUND.play();
		}

		/**
		 * 
		 */
		private function handleSampleData(event : SampleDataEvent) : void
		{
			// Latency in milliSeconds. This can vary and needs to be calculated each SampleDataEvent.
			// The formula is ((SampleDataEvent.position / 44.1) - SoundChannelObject.position)
			if(_soundChannel) _soundLatency = (event.position * 2.267573696145e-02) - _soundChannel.position;
			
			clearBuffer();
			
			_engine.process(BUFFER);

			var l : Vector.<Number> = BUFFER[0]; // left channel
			var r : Vector.<Number> = BUFFER[1]; // right channel
			var i : int = 0;
			var n : int = BUFFER_SIZE;
			
			for(;i < n; ++i)
			{
				event.data.writeFloat(l[i]);
				event.data.writeFloat(r[i]);
			}
		}

		/**
		 * 
		 */
		private function clearBuffer() : void
		{
			var l : Vector.<Number> = BUFFER[0];
			var r : Vector.<Number> = BUFFER[1];
			
			for(var i : int = 0 ;i < BUFFER_SIZE; ++i)
			{
				l[i] = 0.0;
				r[i] = 0.0;
			}
		}
	}
}
