package com.hisschemoller.model 
{
	import com.hisschemoller.model.vo.SamplerPresetVO;
	
	import org.puremvc.as3.interfaces.IProxy;
	import org.puremvc.as3.patterns.proxy.Proxy;
	
	import flash.display.BitmapData;
	import flash.display.Shape;
	import flash.utils.Timer;	
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class WaveDisplayProxy extends Proxy implements IProxy 
	{
		public static const NAME : String = 'WaveDisplayProxy';
		public static const GRAPHIC_UPDATED : String = 'GraphicUpdated';
		public static const ZOOM_UPDATED : String = 'zoomUpdated';
		private var _presetVO : SamplerPresetVO;
		private var _updateIntervalTimer : Timer = new Timer(50, 1);
		private var _drawShape : Shape = new Shape();
		private var _minSamplesPerPixel : Number = 1;
		private var _maxSamplesPerPixel : Number = 1;
		private var _currentSamplesPerPixel : Number = 1;
		private var _minVerticalZoomAmount : Number = 1;
		private var _maxVerticalZoomAmount : Number = 10;
		private var _pollingInterval : Number = 1;
		private var _numPolledSamples : int;
		private var _bytesToSkip : int;
		private var _displayWidth : int = 430;
		private var _displayHeight : int = 160;
		private var _displayMiddle : int;
		private var _firstSampleInView : int;
		private var _numSamplesInView : Number;
		private var _numSamplesTotal : int;
		private var _verticalZoomAmount : Number;
		private var _filledGraphicTreshold : int = 20;
		// Drag to move and zoom variables
		private var _dragXAtMouseDown : Number;
		private var _previousMouseY : Number;
		private var _zoomCenterPosition : Number;
		private var _sampleAtZoomCenter : int;

		public function WaveDisplayProxy()
		{
			super(NAME);
		}

		/**
		 * Called by the Model when the Proxy is registered
		 */
		override public function onRegister() : void
		{
			_displayMiddle = _displayHeight / 2;
			_verticalZoomAmount = _displayMiddle;
		}

		/**
		 * @param presetVO : SamplerPresetVO object with the sound to display
		 */
		public function changeSamplerPreset(presetVO : SamplerPresetVO) : void 
		{
			_presetVO = presetVO;
			_presetVO.soundGraphic = new BitmapData(_displayWidth, _displayHeight, true, 0xFF0000);
			
			_numSamplesTotal = _presetVO.soundData.numSamplesTotal;
			_maxSamplesPerPixel = Number(_numSamplesTotal) / Number(_displayWidth);
			_firstSampleInView = 0;
			_zoomCenterPosition = 0.5;
			_sampleAtZoomCenter = _firstSampleInView + Math.round(_numSamplesTotal * _zoomCenterPosition);
			
			currentSamplesPerPixel = _maxSamplesPerPixel;
			verticalZoomLevel = 0;
			
			forceUpdate();
		}

		/**
		 * Resets the state of the proxy.
		 */
		public function clear() : void
		{
			_maxSamplesPerPixel = 1;
			_currentSamplesPerPixel = 1;
			_pollingInterval = 1;
			_firstSampleInView = 0;
			_numSamplesInView = 0;
			_numSamplesTotal = 0;
			_zoomCenterPosition = 0;
			_sampleAtZoomCenter = 0;
			_verticalZoomAmount = _displayMiddle;
			_presetVO = null;
			
			updateGraphic();
		}
		
		/**
		 * 
		 */
		public function forceUpdate() : void
		{
			_updateIntervalTimer.stop();
			updateGraphic();
		}

		/**
		 * @description: The mouse is pressed in the wave display area.
		 * @param xAtMouseDown : mouseX at MOUSE_DOWN
		 * @param yAtMouseDown : mouseY at MOUSE_DOWN
		 */
		public function initializeDrag(xAtMouseDown : Number, yAtMouseDown : Number) : void
		{
			_dragXAtMouseDown = xAtMouseDown;
			_previousMouseY = yAtMouseDown;
			
			_zoomCenterPosition = xAtMouseDown / _displayWidth;
			_sampleAtZoomCenter = _firstSampleInView + Math.round(_numSamplesInView * _zoomCenterPosition);
		}

		/**
		 * @description: The mouse is dragged in the wave display area. In response
		 * 		the waveform should be moved according to the horizontal mouse movement
		 * 		and zoomed in or out following the vertical mouse movement.
		 * @param newDragX : mouseX at MOUSE_MOVE
		 * @param newDragY : mouseY at MOUSE_MOVE
		 */
		public function updateDrag(newDragX : Number, newDragY : Number) : void
		{
			/** The distance the mouse was dragged. */
			var distanceX : Number = newDragX - _dragXAtMouseDown;
			var distanceY : Number = newDragY - _previousMouseY;
			_previousMouseY = newDragY;
			
			/** The new zoom level. */
			_currentSamplesPerPixel -= (distanceY * _currentSamplesPerPixel * 0.01);
			_currentSamplesPerPixel = Math.max(_minSamplesPerPixel, Math.min(_currentSamplesPerPixel, _maxSamplesPerPixel));
			_numSamplesInView = _currentSamplesPerPixel * _displayWidth;
			
			/** The new position. */
			_zoomCenterPosition = (_dragXAtMouseDown + distanceX) / _displayWidth;
			_firstSampleInView = _sampleAtZoomCenter - (_numSamplesInView * _zoomCenterPosition);
			_firstSampleInView = Math.max(0, Math.min(_firstSampleInView, _numSamplesTotal - _numSamplesInView));
			
			/** Polling stuff. */
			_pollingInterval = Math.max(1, _currentSamplesPerPixel / 20);
			_numPolledSamples = int(_numSamplesInView / _pollingInterval);
			_bytesToSkip = Math.round(_pollingInterval - 1) * AudioDriverProxy.BYTES_PER_SAMPLE;
			
			facade.sendNotification(WaveDisplayProxy.ZOOM_UPDATED);
			
			updateGraphic();
		}

		/**
		 * @return : horizontal zoom as Number between 0 and 1.
		 */
		public function get horizontalZoomLevel() : Number
		{
			return (_currentSamplesPerPixel - _minSamplesPerPixel) / (_maxSamplesPerPixel - _minSamplesPerPixel);
		}

		/**
		 * @param inValue : Number between 0 and 1.
		 */
		public function set horizontalZoomLevel(value : Number) : void
		{
			// Scale value for more precision when zoomed in.
			var valueScaled : Number = value * value;
			
			currentSamplesPerPixel = _minSamplesPerPixel + ((_maxSamplesPerPixel - _minSamplesPerPixel) * valueScaled);
		}

		/**
		 * 
		 */
		public function get minSamplesPerPixel() : Number
		{
			return _minSamplesPerPixel;
		}

		/**
		 * 
		 */
		public function get maxSamplesPerPixel() : Number
		{
			return _maxSamplesPerPixel;
		}

		/**
		 * 
		 */
		public function get currentSamplesPerPixel() : Number
		{
			return _currentSamplesPerPixel;
		}

		/**
		 * Sets the zoom level of the display. Measured in how many samples one horizontal pixel of the display represents.
		 * @param samplesPerPixel : How many samples one horizontal pixel of the display represents.
		 */
		public function set currentSamplesPerPixel(samplesPerPixel : Number) : void
		{
			if(samplesPerPixel == _currentSamplesPerPixel) return;
			
			// The new zoom level.
			_currentSamplesPerPixel = Math.max(_minSamplesPerPixel, Math.min(samplesPerPixel, _maxSamplesPerPixel));
			_numSamplesInView = _currentSamplesPerPixel * _displayWidth;
			
			// The new position.
			_firstSampleInView = _sampleAtZoomCenter - (_numSamplesInView * _zoomCenterPosition);
			_firstSampleInView = Math.max(0, Math.min(_firstSampleInView, _numSamplesTotal - _numSamplesInView));
			
			// Polling stuff.
			_pollingInterval = Math.max(1, _currentSamplesPerPixel / 20);
			_numPolledSamples = int(_numSamplesInView / _pollingInterval);
			_bytesToSkip = Math.round(_pollingInterval - 1) * AudioDriverProxy.BYTES_PER_SAMPLE;
			
			updateGraphic();
		}

		/**
		 * 
		 */
		public function get firstSampleInView() : int
		{
			return _firstSampleInView;
		}

		/**
		 * 
		 */
		public function set firstSampleInView(firstSample : int) : void
		{
			setFirstSampleInView(firstSample);
		}

		/**
		 * 
		 */
		public function get numSamplesInView() : int
		{
			return _numSamplesInView;
		}

		/**
		 * 
		 */
		public function get numSamplesTotal() : int
		{
			return _numSamplesTotal;
		}

		/**
		 * @param value : Number between 0 and 1
		 */
		public function set verticalZoomLevel(value : Number) : void
		{
			_verticalZoomAmount = _displayMiddle * (_minVerticalZoomAmount + ((_maxVerticalZoomAmount - _minVerticalZoomAmount) * value));
			
			updateGraphic();
		}

		/**
		 * Repositions the displayed part of the wave by setting the first visible sample at the left edge of the display.
		 * @param firstSample : The first visible sample at the left edge of the display.
		 * @param doUpdate : True if the display should be updated after the setting was changed.
		 */
		public function setFirstSampleInView(firstSample : int, doUpdate : Boolean = true) : void
		{
			if(firstSample == _firstSampleInView) return;
			
			_firstSampleInView = firstSample;
			_firstSampleInView = Math.max(0, Math.min(_firstSampleInView, _numSamplesTotal - _numSamplesInView));
			
			if(doUpdate) updateGraphic();
		}

		/**
		 * Redraws the display.
		 */
		private function updateGraphic() : void
		{
			// delay between updates
			if(_updateIntervalTimer.running) return;
			_updateIntervalTimer.start();
			
			_drawShape.graphics.clear();
			_drawShape.graphics.lineStyle(1.2, 0x990000);
			_drawShape.graphics.moveTo(0, _displayMiddle);
			
			_presetVO.soundData.byteArray.position = int(_firstSampleInView * AudioDriverProxy.BYTES_PER_SAMPLE);
			
			var samplePosition : Number = 0;
			var nextPixelSamplePosition : Number = _currentSamplesPerPixel;
			var pixelIndex : uint = 0;
			var positiveWaveHeight : Number = 0.0;
			var negativeWaveHeight : Number = 0.0;
			
			for(var i : int = 0;i < _numPolledSamples; ++i)
			{
				// Read the left channel.
				var amplitude : Number = _presetVO.soundData.byteArray.readFloat();
				// Skip the right channel.
				_presetVO.soundData.byteArray.readFloat();
				// And jump to the next position to poll.
				_presetVO.soundData.byteArray.position = int(_firstSampleInView + samplePosition) * AudioDriverProxy.BYTES_PER_SAMPLE;
				
				// store only highest positive and negative amplitude 
				if(amplitude > 0) positiveWaveHeight = Math.max(positiveWaveHeight, amplitude);
				else negativeWaveHeight = Math.min(negativeWaveHeight, amplitude);
				
				// If all sample polling for the current pixel is done draw the result.
				if(samplePosition >= nextPixelSamplePosition)
				{
					// Draw: Filled waves or a line if the graphic is zoomed in far enough.
					if(_currentSamplesPerPixel > _filledGraphicTreshold)
					{
						_drawShape.graphics.moveTo(pixelIndex, _displayMiddle + (negativeWaveHeight * _verticalZoomAmount));
						_drawShape.graphics.lineTo(pixelIndex, _displayMiddle + (positiveWaveHeight * _verticalZoomAmount));
					}
					else
					{
						if(positiveWaveHeight > (negativeWaveHeight * -1))
						{
							_drawShape.graphics.lineTo(pixelIndex, _displayMiddle + (positiveWaveHeight * _verticalZoomAmount));
						}
						else
						{
							_drawShape.graphics.lineTo(pixelIndex, _displayMiddle + (negativeWaveHeight * _verticalZoomAmount));
						}
					}
					
					// Prepare for next pixel / loop iteration.
					nextPixelSamplePosition += _currentSamplesPerPixel;
					positiveWaveHeight = 0;
					negativeWaveHeight = 0;
					pixelIndex ++;
				}
				
				samplePosition += _pollingInterval;
			}
			
			// Clear and redraw the bitmapData
			_presetVO.soundGraphic.fillRect(_presetVO.soundGraphic.rect, 0);
			_presetVO.soundGraphic.draw(_drawShape);
			
			sendNotification(WaveDisplayProxy.GRAPHIC_UPDATED, _presetVO.soundGraphic);
		}
	}
}
