package com.hisschemoller.model.audio.generators 
{
	import com.hisschemoller.model.AudioDriverProxy;
	import com.hisschemoller.model.audio.IAudioProcessor;
	
	import flash.utils.ByteArray;	
	/**
	 * Plays part of an MP3 file (a slice) from _startSampleIndex to _endSampleIndex.
	 * 
	 * @author Wouter Hisschemöller
	 * (c) Apr 30, 2009
	 */
	public class MP3SliceExtractor implements IAudioProcessor 
	{
		private const BYTE_ARRAY : ByteArray = new ByteArray();
		private var _sourceByteArray : ByteArray;
		private var _deltaSamples : int;
		private var _startSampleIndex : int;
		private var _endSampleIndex : int;
		private var _currentSampleIndex : int;
		private var _relativeSampleIndex : int;
		private var _attackDuration : int = 100;
		private var _decayDuration : int = 100;
		private var _gain : Number = 1;
		/**
		 * @param deltaSamples : offset into the buffer where to start writing
		 * @param sourceByteArray : the source ByteArray from which to read samples.
		 * @param startSampleIndex : the sample position where the slice starts
		 * @param endSampleIndex : the sample position where the slice ends
		 */
		public function MP3SliceExtractor(deltaSamples : int, sourceByteArray : ByteArray, startSampleIndex : int, endSampleIndex : int)
		{
			_deltaSamples = deltaSamples;
			_sourceByteArray = sourceByteArray;
			_startSampleIndex = startSampleIndex;
			_endSampleIndex = endSampleIndex;
			_currentSampleIndex = startSampleIndex;
		}

		/**
		 * @param b uffer : stereo buffer of AudioDriver.BUFFER_SIZE number of samples
		 * @return Boolean : true if the playback is done and this generator can be removed
		 */
		public function process(buffer : Vector.<Vector.<Number>>) : Boolean
		{
			/** if the end of the slice is reached within the current buffer, adjust the samples to read. */
			if(_currentSampleIndex + AudioDriverProxy.BUFFER_SIZE <= _endSampleIndex)
			{
				var numSamples : int = AudioDriverProxy.BUFFER_SIZE;
			}
			else
			{
				numSamples = _endSampleIndex - _currentSampleIndex;
			}
			
			BYTE_ARRAY.position = 0;
			BYTE_ARRAY.writeBytes(_sourceByteArray, _currentSampleIndex * AudioDriverProxy.BYTES_PER_SAMPLE, numSamples * AudioDriverProxy.BYTES_PER_SAMPLE);
			BYTE_ARRAY.position = 0;
			
			var l : Vector.<Number> = buffer[0];
			var r : Vector.<Number> = buffer[1];
			
			for(var i : int = _deltaSamples ;i < numSamples; ++i)
			{
				/** A short attack to remove clicks at the start of the slice */
				if(_relativeSampleIndex + i < _attackDuration)
				{
					_gain = (_relativeSampleIndex + i) / _attackDuration;
				}
				
				/** A short decay to remove clicks at the end of the slice. */
				if(_endSampleIndex - _relativeSampleIndex + i < _decayDuration)
				{ 
					_gain = (_endSampleIndex - _relativeSampleIndex + i) / _decayDuration;
				}
				
				l[i] += BYTE_ARRAY.readFloat() * _gain;
				r[i] += BYTE_ARRAY.readFloat() * _gain;
			}
			
			/** If slice is done, fill the rest of the buffer with silence. */
			if(numSamples < AudioDriverProxy.BUFFER_SIZE)
			{
				numSamples = AudioDriverProxy.BUFFER_SIZE;

				for(;i < numSamples; ++i)
				{
					l[i] += 0.0;
					l[i] += 0.0;
				}
			}
			
			_currentSampleIndex += AudioDriverProxy.BUFFER_SIZE - _deltaSamples;
			_relativeSampleIndex = _currentSampleIndex - _startSampleIndex;
			_deltaSamples = 0;
			
			if(_currentSampleIndex >= _endSampleIndex) return true;
			
			return false;
		}
	}
}
