package com.hisschemoller.view.components 
{
	import assets.SampleEditorAsset;
	
	import com.hisschemoller.view.components.base.Component;
	import com.hisschemoller.view.components.base.LocatorBar;
	import com.hisschemoller.view.components.base.PushButton;
	
	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	import flash.events.MouseEvent;	
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class SampleEditor extends Component 
	{
		public static const START_PLAY_LOCATOR_CHANGE : String = "startPlayLocatorChange";
		public static const END_PLAY_LOCATOR_CHANGE : String = "endPlayLocatorChange";
		public static const START_LOOP_LOCATOR_CHANGE : String = "startLoopLocatorChange";
		public static const END_LOOP_LOCATOR_CHANGE : String = "endLoopLocatorChange";
		public static const REVERSE_CHANGE : String = "reverseChange";
		public static const TRANSPOSE_CHANGE : String = "transposeChange";
		private var _playLocatorBar : PlayLocatorBar;
		//private var _loopLocatorBar : LoopLocatorBar;
		private var _reverseButton : ToggleButton;
		private var _startLabel : AlignRightLabel;
		private var _endLabel : AlignRightLabel;
		private var _transposeSlider : TransposeSlider;
		private var _transposeLabel : AlignRightLabel;

		public function SampleEditor(container : DisplayObjectContainer)
		{
			super(container, new SampleEditorAsset());
		}

		/**
		 * Update locator position.
		 */
		public function updateLocators(startValue : Number, endValue : Number, startIndex : int, endIndex : int) : void
		{
			_playLocatorBar.startValue = startValue;
			_playLocatorBar.endValue = endValue;
			
			_startLabel.setText(startIndex.toString());
			_endLabel.setText(endIndex.toString());
		}

		/**
		 * Return locator position
		 */
		public function getLocatorValue(locator : String) : Number
		{
			switch(locator)
			{
				case START_PLAY_LOCATOR_CHANGE:
					return _playLocatorBar.startValue;
					break;
				case END_PLAY_LOCATOR_CHANGE:
					return _playLocatorBar.endValue;
					break;
				case START_LOOP_LOCATOR_CHANGE:
					//
					break;
				case END_LOOP_LOCATOR_CHANGE:
					//
					break;
			}
			
			return NaN;
		}
		
		/**
		 * 
		 */
		public function set transpose(value : Number) : void
		{
			_transposeSlider.value = value;
			_transposeLabel.setText(value.toFixed(2));
		}
		
		/**
		 * 
		 */
		public function get transpose() : Number
		{
			return _transposeSlider.value;
		}

		/**
		 * 
		 */
		public function set reverse(value : Boolean) : void
		{
			_reverseButton.selected = value;
		}

		/**
		 * 
		 */
		public function get reverse() : Boolean
		{
			return _reverseButton.selected;
		}

		/**
		 * 
		 */
		override protected function addChildren() : void
		{
			_playLocatorBar = new PlayLocatorBar(asset.tPlayLocatorBarAsset);
			_playLocatorBar.addEventListener(LocatorBar.START_LOCATOR_CHANGE, handlePlayLocatorEvent);
			_playLocatorBar.addEventListener(LocatorBar.END_LOCATOR_CHANGE, handlePlayLocatorEvent);
			
			_startLabel = new AlignRightLabel(asset.tStartLabel);
			_endLabel = new AlignRightLabel(asset.tEndLabel);
			
			_reverseButton = new ToggleButton(asset.tReverseButton);
			_reverseButton.toggle = true;
			_reverseButton.addEventListener(MouseEvent.CLICK, reverseClickHandler);
			
			_transposeSlider = new TransposeSlider(asset.tTransposeSlider);
			_transposeSlider.minimum = -12;
			_transposeSlider.maximum = 12;
			_transposeSlider.snapInterval = 1;
			_transposeSlider.addEventListener(Event.CHANGE, transposeSliderChangeHandler);
			
			_transposeLabel = new AlignRightLabel(asset.tTransposeLabel);
		}

		/**
		 * 
		 */
		private function handlePlayLocatorEvent(event : Event) : void
		{
			switch(event.type)
			{
				case LocatorBar.START_LOCATOR_CHANGE:
					dispatchEvent(new Event(SampleEditor.START_PLAY_LOCATOR_CHANGE));
					break;
				case LocatorBar.END_LOCATOR_CHANGE:
					dispatchEvent(new Event(SampleEditor.END_PLAY_LOCATOR_CHANGE));
					break;
			}
		}

		/**
		 * 
		 */
		private function reverseClickHandler(event : MouseEvent) : void
		{
			dispatchEvent(new Event(SampleEditor.REVERSE_CHANGE));
		}

		/**
		 * 
		 */
		private function transposeSliderChangeHandler(event : Event) : void
		{
			dispatchEvent(new Event(SampleEditor.TRANSPOSE_CHANGE));
		}

		/**
		 * 
		 */
		private function get asset() : SampleEditorAsset
		{
			return _asset as SampleEditorAsset;
		}
	}
}
