package com.hisschemoller.view.components 
{
	import assets.WaveDisplayAsset;
	
	import com.hisschemoller.view.components.base.Component;
	
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextFieldAutoSize;	
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class WaveDisplay extends Component 
	{
		public static const HORIZONTAL_ZOOM_CHANGE : String = "horizontalZoomChange";
		public static const VERTICAL_ZOOM_CHANGE : String = "verticalZoomChange";
		public static const POSITION_CHANGE : String = "positionChange";
		public static const MOUSE_DRAG : String = "mouseDrag";
		private var _hZoomSlider : HSlider;
		private var _vZoomSlider : HSlider;
		private var _firstLabel : AlignLeftLabel;
		private var _lastLabel : AlignRightLabel;
		private var _samplesPerPixelLabel : AlignCenterLabel;
		private var _positionScrollBar : HScrollBar;
		private var _waveBitmap : Bitmap;
		private var _mouseIsDown : Boolean;

		public function WaveDisplay(container : DisplayObjectContainer)
		{
			super(container, new WaveDisplayAsset());
		}

		/**
		 * 
		 */
		public function updateGraphic(bitmapData : BitmapData) : void
		{
			_waveBitmap.bitmapData = bitmapData;
		}

		/**
		 * 
		 */
		public function updateHZoomSlider(value : Number) : void
		{
			_hZoomSlider.value = value;
		}

		/**
		 * 
		 */
		public function updateVZoomSlider(value : Number) : void
		{
			_vZoomSlider.value = value;
		}

		/**
		 * 
		 */
		public function updatePosition(numSamplesInView : int, numSamplesTotal : int, firstSampleInView : int, currentSamplesPerPixel : Number) : void
		{
			_positionScrollBar.pageSize = numSamplesInView;
			_positionScrollBar.minScrollPosition = 0;
			_positionScrollBar.maxScrollPosition =numSamplesTotal - numSamplesInView;
			_positionScrollBar.pageScrollSize = numSamplesInView * 0.5;
			_positionScrollBar.scrollPosition = firstSampleInView;
			
			_firstLabel.setText(firstSampleInView.toString());
			_lastLabel.setText((firstSampleInView + numSamplesInView).toString());
			_samplesPerPixelLabel.setText(currentSamplesPerPixel.toFixed(2));
		}

		/**
		 * 
		 */
		public function get hZoomSliderValue() : Number
		{
			return _hZoomSlider.value;
		}

		/**
		 * 
		 */
		public function get vZoomSliderValue() : Number
		{
			return _vZoomSlider.value;
		}

		/**
		 * 
		 */
		public function get positionScrollBarPosition() : Number
		{
			return _positionScrollBar.scrollPosition;
		}

		/**
		 * 
		 */
		public function get graphicMouseX() : Number
		{
			return _waveBitmap.mouseX;
		}

		/**
		 * 
		 */
		public function get graphicMouseY() : Number
		{
			return _waveBitmap.mouseY;
		}

		/**
		 * 
		 */
		override protected function addChildren() : void
		{
			_hZoomSlider = new HSlider(asset.tHZoomSliderAsset);
			_hZoomSlider.addEventListener(Event.CHANGE, handleHZoomSliderChange);
			
			_vZoomSlider = new HSlider(asset.tVZoomSliderAsset);
			_vZoomSlider.addEventListener(Event.CHANGE, handleVZoomSliderChange);
			
			_firstLabel = new AlignLeftLabel(asset.tFirstLabel);
			_firstLabel.setAutoSize(TextFieldAutoSize.LEFT);
			
			_lastLabel = new AlignRightLabel(asset.tLastLabel);
			_lastLabel.setAutoSize(TextFieldAutoSize.RIGHT);
			
			_samplesPerPixelLabel = new AlignCenterLabel(asset.tSPPLabel);
			_samplesPerPixelLabel.setAutoSize(TextFieldAutoSize.CENTER);
			
			_positionScrollBar = new HScrollBar(asset.tPositionScrollBarAsset);
			_positionScrollBar.addEventListener(Event.CHANGE, handlePositionScrollBarChange);
			
			_waveBitmap = new Bitmap();
			_waveBitmap.x = asset.tWaveDisplayAsset.x;
			_waveBitmap.y = asset.tWaveDisplayAsset.y;
			asset.addChild(_waveBitmap);
			
			asset.tWaveDisplayAsset.doubleClickEnabled = true;
			asset.tWaveDisplayAsset.addEventListener(MouseEvent.MOUSE_DOWN, graphicMouseDownHandler);
			asset.tWaveDisplayAsset.addEventListener(MouseEvent.DOUBLE_CLICK, graphicDoubleClickHandler);
		}

		/**
		 * 
		 */
		private function handleHZoomSliderChange(event : Event) : void
		{
			dispatchEvent(new Event(WaveDisplay.HORIZONTAL_ZOOM_CHANGE));
		}

		/**
		 * 
		 */
		private function handleVZoomSliderChange(event : Event) : void
		{
			dispatchEvent(new Event(WaveDisplay.VERTICAL_ZOOM_CHANGE));
		}

		/**
		 * 
		 */
		private function handlePositionScrollBarChange(event : Event) : void
		{
			dispatchEvent(new Event(WaveDisplay.POSITION_CHANGE));
		}

		/**
		 * 
		 */
		private function graphicMouseDownHandler(event : MouseEvent) : void
		{
			_mouseIsDown = true;
			
			asset.stage.addEventListener(MouseEvent.MOUSE_MOVE, handleGraphicMouseMove);
			asset.stage.addEventListener(MouseEvent.MOUSE_UP, handleGraphicMouseUp);
			asset.stage.addEventListener(Event.MOUSE_LEAVE, handleGraphicMouseUp);
			
			dispatchEvent(event);
		}

		/**
		 * 
		 */
		private function handleGraphicMouseMove(event : MouseEvent) : void
		{
			if(_mouseIsDown)
			{
				dispatchEvent(new Event(WaveDisplay.MOUSE_DRAG));
			}
		}

		/**
		 * 
		 */
		private function handleGraphicMouseUp(event : Event) : void
		{
			_mouseIsDown = false;
			
			asset.stage.removeEventListener(MouseEvent.MOUSE_MOVE, handleGraphicMouseMove);
			asset.stage.removeEventListener(MouseEvent.MOUSE_UP, handleGraphicMouseUp);
			asset.stage.removeEventListener(Event.MOUSE_LEAVE, handleGraphicMouseUp);
		}

		/**
		 * 
		 */
		private function graphicDoubleClickHandler(event : MouseEvent) : void
		{
			dispatchEvent(event);
		}

		/**
		 * 
		 */
		private function get asset() : WaveDisplayAsset
		{
			return _asset as WaveDisplayAsset;
		}
	}
}
