package com.hisschemoller.view.components.base
{
	import assets.EndLocator;	
	import assets.StartLocator;	
	
	import com.hisschemoller.view.components.base.Component;
	
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Rectangle;	
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class LocatorBar extends Component 
	{
		public static const START_LOCATOR_CHANGE : String = "startLocatorChange";
		public static const END_LOCATOR_CHANGE : String = "endLocatorChange";
		protected var _track : MovieClip;
		protected var _fill : MovieClip;
		protected var _startLocator : StartLocator;
		protected var _endLocator : EndLocator;
		protected var _activeLocator : MovieClip;
		protected var _minimum : Number = 0;
		protected var _maximum : Number = 1;
		protected var _startValue : Number = 0.25;
		protected var _endValue : Number = 0.75;
		protected var _isDragging : Boolean;
		
		public function LocatorBar(inAsset : MovieClip)
		{
			super(null, inAsset);
		}
		
		/**
		 * 
		 */
		override public function draw() : void
		{
			if(_startValue > _endValue) _startValue = _endValue;
			if(_endValue < _startValue) _endValue = _startValue;
			
			if(!_isDragging)
			{
				_startLocator.x = _track.x + (_startValue * _track.width);
				_endLocator.x = _track.x + (_endValue * _track.width);
			}
			
			_fill.x = Math.max(_minimum, _startLocator.x);
			_fill.width = _endLocator.x - _fill.x;
			
			if((_startValue < _minimum) || (_startValue > _maximum)) _startLocator.visible = false;
			else _startLocator.visible = true;
			
			if((_endValue < _minimum) || (_endValue > _maximum)) _endLocator.visible = false;
			else _endLocator.visible = true;
			
			if((_endValue < _minimum) || (_startValue > _maximum)) _fill.visible = false;
			else _fill.visible = true;
		}
		
		/**
		 * 
		 */
		public function get startValue() : Number
		{
			return _startValue;
		}

		/**
		 * @param inValue : new value for the start locator
		 */
		public function set startValue(inValue : Number) : void
		{
			if(_isDragging) return;
			
			_startValue = inValue;
			
			invalidate();
		}
		
		/**
		 * 
		 */
		public function get endValue() : Number
		{
			return _endValue;
		}
		
		/**
		 * @param inValue : new value for the end locator
		 */
		public function set endValue(inValue : Number) : void
		{
			if(_isDragging) return;
			
			_endValue = inValue;
			
			invalidate();
		}
		
		/**
		 * 
		 */
		override protected function init() : void
		{
			super.init();
		}

		/**
		 * Creates and adds the child display objects of this component.
		 */
		override protected function addChildren() : void
		{
			new PushButton(_startLocator);
			new PushButton(_endLocator);
			
			_startLocator.stop();
			_startLocator.hitArea = _startLocator.tHitArea;
			_startLocator.addEventListener(MouseEvent.MOUSE_DOWN, handleLocatorMouseDown);
			
			_endLocator.stop();
			_endLocator.hitArea = _endLocator.tHitArea;
			_endLocator.addEventListener(MouseEvent.MOUSE_DOWN, handleLocatorMouseDown);
		}

		/**
		 * Internal mouseDown handler. Mouse pressed on locator
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function handleLocatorMouseDown(inEvent : MouseEvent) : void
		{
			_isDragging = true;
			
			_asset.stage.addEventListener(MouseEvent.MOUSE_MOVE, handleLocatorMouseMove);
			_asset.stage.addEventListener(MouseEvent.MOUSE_UP, handleLocatorMouseUp);
			_asset.stage.addEventListener(Event.MOUSE_LEAVE, handleLocatorMouseUp);
			
			_activeLocator = inEvent.target as MovieClip;
			
			if(_activeLocator == _startLocator) _startLocator.startDrag(false, new Rectangle(_track.x, _track.y, Math.min((_endLocator.x - _track.x), _track.width), 0));
			else if(_activeLocator == _endLocator) _endLocator.startDrag(false, new Rectangle(Math.max(_track.x, _startLocator.x), _track.y, (_track.width - _startLocator.x + _track.x), 0));
		}

		/**
		 * 
		 */
		private function handleLocatorMouseMove(inEvent : MouseEvent) : void
		{
			if(_isDragging)
			{
				draw();
				
				if(_activeLocator == _startLocator)
				{
					_startValue = _minimum + ((_maximum - _minimum) * ((_startLocator.x - _track.x) / _track.width));
					dispatchEvent(new Event(LocatorBar.START_LOCATOR_CHANGE));
				}
				else if(_activeLocator == _endLocator)
				{
					_endValue = _minimum + ((_maximum - _minimum) * ((_endLocator.x - _track.x) / _track.width));
					dispatchEvent(new Event(LocatorBar.END_LOCATOR_CHANGE));
				}
			}
		}

		/**
		 * 
		 */
		private function handleLocatorMouseUp(inEvent : Event) : void
		{
			_isDragging = false;
			
			_asset.stage.removeEventListener(MouseEvent.MOUSE_MOVE, handleLocatorMouseMove);
			_asset.stage.removeEventListener(MouseEvent.MOUSE_UP, handleLocatorMouseUp);
			_asset.stage.removeEventListener(Event.MOUSE_LEAVE, handleLocatorMouseUp);
			
			_activeLocator.stopDrag();
			_activeLocator = null;
		}
	}
}
