package com.hisschemoller.view.components.base 
{
	import com.hisschemoller.view.components.base.Component;
	
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.geom.Rectangle;
	import flash.utils.Timer;	
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class ScrollBar extends Component 
	{
		public static const HORIZONTAL : String = "horizontal";
		public static const VERTICAL : String = "vertical";
		protected var _orientation : String;
		protected var _track : MovieClip;
		protected var _thumb : MovieClip;
		protected var _pageSize : Number;
		protected var _minScrollPosition : Number;
		protected var _maxScrollPosition : Number;
		protected var _pageScrollSize : Number;
		protected var _scrollPosition : Number = 0;
		protected var _isScrolling : Boolean = false;
		protected var _trackScrollTimer : Timer;
		protected var _pageScrollDirection : int;

		public function ScrollBar(asset : MovieClip, orientation : String = ScrollBar.HORIZONTAL)
		{
			_orientation = orientation;
			
			super(null, asset);
		}

		/**
		 * Convenience method to set the four main properties in one shot.
		 * @param pageSize : Number which represents the size of one page.
		 * @param minScrollPosition : Number which represents the bottom of the scrolling range.
		 * @param maxScrollPosition : Number which represetns the top of the scrolling range.
		 * @param pageScrollSize : Number which represents the increment to move when the scroll track is pressed.
		 * @param scrollPosition : The current value of the slider.
		 */
		public function setScrollProperties(pageSize : Number, minScrollPosition : Number, maxScrollPosition : Number, pageScrollSize : Number = 0, scrollPosition : Number = 0) : void
		{
			this.pageSize = pageSize;
			this.minScrollPosition = minScrollPosition;
			this.maxScrollPosition = maxScrollPosition;
			this.pageScrollSize = pageScrollSize;
			this.scrollPosition = scrollPosition;
		}

		/**
		 * Sets / gets the size of one page.
		 */
		public function set pageSize(pageSize : Number) : void
		{
			_pageSize = pageSize;
			correctValue();
			invalidate();
		}

		public function get pageSize() : Number
		{
			return _pageSize;
		}

		/**
		 * Sets / gets the increment to move when the scroll track is pressed.
		 */
		public function set pageScrollSize(pageScrollSize : Number) : void
		{
			_pageScrollSize = pageScrollSize;
		}

		public function get pageScrollSize() : Number
		{
			return _pageScrollSize;
		}

		/**
		 * Sets / gets the bottom of the scrolling range.
		 */
		public function set minScrollPosition(minScrollPosition : Number) : void
		{
			_minScrollPosition = minScrollPosition;
			correctValue();
			invalidate();
		}

		public function get minScrollPosition() : Number
		{
			return _minScrollPosition;
		}

		/**
		 * Sets / gets the top of the scrolling range.
		 */
		public function set maxScrollPosition(maxScrollPosition : Number) : void
		{
			_maxScrollPosition = maxScrollPosition;
			correctValue();
			invalidate();
		}

		public function get maxScrollPosition() : Number
		{
			return _maxScrollPosition;
		}

		/**
		 * Sets / gets the current value of this slider.
		 */
		public function set scrollPosition(position : Number) : void
		{
			if(_isScrolling) return;
			
			_scrollPosition = position;
			correctValue();
			invalidate();
		}

		public function get scrollPosition() : Number
		{
			return _scrollPosition;
		}

		/**
		 * Adjusts position of thumb when _scrollPosition, _minScrollPosition, _maxScrollPosition or _pageSize have changed.
		 * TODO: Should also be called when slider is resized.
		 */
		override public function draw() : void
		{
			if(_orientation == ScrollBar.HORIZONTAL)
			{
				_thumb.width = Math.max(20, (_pageSize / ((_maxScrollPosition + _pageSize) - _minScrollPosition)) * _track.width);
				_thumb.x = (_scrollPosition - _minScrollPosition) / (_maxScrollPosition - _minScrollPosition) * (_track.width - _thumb.width);
			}
			else
			{
				_thumb.height = Math.max(20, (_pageSize / ((_maxScrollPosition + _pageSize) - _minScrollPosition)) * _track.height);
				_thumb.y = (_scrollPosition - _minScrollPosition) / (_maxScrollPosition - _minScrollPosition) * (_track.height - _thumb.height);
			}
		}
		
		/**
		 * 
		 */
		override protected function init() : void
		{
			super.init();
			
			_trackScrollTimer = new Timer(200, 1);
			_trackScrollTimer.addEventListener(TimerEvent.TIMER, handleTrackScrollTimer);
		}

		/**
		 * Adjusts value to be within minimum and maximum.
		 */
		protected function correctValue() : void
		{
			_scrollPosition = Math.min(_scrollPosition, _maxScrollPosition);
			_scrollPosition = Math.max(_scrollPosition, _minScrollPosition);
		}
		
		/**
		 * scroll one pageSize at a time (when track is clicked)
		 */
		protected function pageScroll() : void
		{
			var delta : Number = _pageScrollSize != 0 ? _pageScrollSize : _pageSize;
			var newPos : Number = _scrollPosition + (_pageScrollDirection * delta);
			
			if (newPos > _maxScrollPosition) newPos = _maxScrollPosition;
			else if (newPos < _minScrollPosition) newPos = _minScrollPosition;
			
			if(newPos != _scrollPosition)
			{
				scrollPosition = newPos;
				
				invalidate();
				dispatchEvent(new Event(Event.CHANGE));
			}
		}

		/**
		 * Handler called when user presses the mouse on the background of the scrollbar, causing the scrollBar to scroll a page at a time.
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function handleTrackMouseDown(event : MouseEvent) : void
		{
			if(_orientation == ScrollBar.HORIZONTAL)
			{
				if(_asset.mouseX < _thumb.x) _pageScrollDirection = -1;
				else _pageScrollDirection = 1;
			}
			else
			{
				if(_asset.mouseY < _thumb.y) _pageScrollDirection = -1;
				else _pageScrollDirection = 1;
			}
			
			_trackScrollTimer.start();
			
			pageScroll();
		}

		/**
		 * Handler called when user releases the mouse on the background of the scrollbar.
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function handleTrackMouseUp(event : MouseEvent) : void
		{
			_trackScrollTimer.reset();
			_trackScrollTimer.delay = 200;
			_trackScrollTimer.repeatCount = 1;
		}

		/**
		 * Internal mouseDown handler. Starts dragging the handle.
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function handleThumbMouseDown(event : MouseEvent) : void
		{
			_isScrolling = true;
			
			_track.stage.addEventListener(MouseEvent.MOUSE_UP, handleStageMouseUp);
			_track.stage.addEventListener(MouseEvent.MOUSE_MOVE, handleStageMouseMove);
			_track.stage.addEventListener(Event.MOUSE_LEAVE, handleStageMouseUp);
			
			if(_orientation == ScrollBar.HORIZONTAL)
			{
				_thumb.startDrag(false, new Rectangle(0, 0, _track.width - _thumb.width, 0));
			}
			else
			{
				_thumb.startDrag(false, new Rectangle(0, 0, 0, _track.height - _thumb.height));
			}
		}

		/**
		 * Internal mouseUp handler. Stops dragging the handle.
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function handleStageMouseUp(event : Event) : void
		{
			_isScrolling = false;
			
			_track.stage.removeEventListener(MouseEvent.MOUSE_UP, handleStageMouseUp);
			_track.stage.removeEventListener(MouseEvent.MOUSE_MOVE, handleStageMouseMove);
			_track.stage.removeEventListener(Event.MOUSE_LEAVE, handleStageMouseUp);
			
			_thumb.stopDrag();
		}

		/**
		 * Internal mouseMove handler for when the handle is being moved.
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function handleStageMouseMove(event : MouseEvent) : void
		{
			var oldPosition : Number = _scrollPosition;
			
			if(_orientation == ScrollBar.HORIZONTAL)
			{
				_scrollPosition = _thumb.x / (_track.width - _thumb.width) * (_maxScrollPosition - _minScrollPosition) + _minScrollPosition;
			}
			else
			{
				_scrollPosition = _thumb.y / (_track.height - _thumb.height) * (_maxScrollPosition - _minScrollPosition) + _minScrollPosition;
			}
			
			if(_scrollPosition != oldPosition)
			{
				dispatchEvent(new Event(Event.CHANGE));
			}
		}

		/**
		 * Repeatedly called by _trackScrollTimer while the mouse is pressed on the scrollbar track.
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function handleTrackScrollTimer(event : TimerEvent) : void
		{
			if(_trackScrollTimer.repeatCount == 1)
			{
				_trackScrollTimer.delay = 100;
				_trackScrollTimer.repeatCount = 0;
			}
			
			pageScroll();
		}
	}
}
