package com.hisschemoller.view.components.base 
{
	import com.hisschemoller.model.enum.InteractionInputType;
	import com.hisschemoller.view.components.base.Component;
	
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;	
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class Slider extends Component 
	{
		public static const HORIZONTAL : String = "horizontal";
		public static const VERTICAL : String = "vertical";
		public static const CLICK_TARGET_TRACK : String = "clickTargetTrack";
		public static const CLICK_TARGET_THUMB : String = "clickTargetThumb";
		protected var _orientation : String;
		protected var _track : MovieClip;
		protected var _thumb : MovieClip;
		protected var _value : Number = 0;
		protected var _minimum : Number = 0;
		protected var _maximum : Number = 1;
		protected var _enableTrackClick : Boolean = true;
		protected var _snapInterval : Number = 0;

		public function Slider(asset : MovieClip, orientation : String = Slider.HORIZONTAL)
		{
			_orientation = orientation;
			
			super(null, asset);
		}

		/**
		 * Convenience method to set the three main properties in one shot.
		 * @param minimum The minimum value of the slider.
		 * @param maximum The maximum value of the slider.
		 * @param value The value of the slider.
		 */
		public function setSliderProperties(minimum : Number, maximum : Number, value : Number) : void
		{
			minimum = minimum;
			maximum = maximum;
			value = value;
		}

		/**
		 * Sets / gets whether or not a click on the background of the slider will move the handler to that position.
		 */
		public function set trackClickEnabled(enabled : Boolean) : void
		{
			_enableTrackClick = enabled;
			invalidate();
		}

		public function get trackClickEnabled() : Boolean
		{
			return _enableTrackClick;
		}

		/**
		 * Sets / gets the current value of this slider.
		 */
		public function set value(value : Number) : void
		{
			doSetValue(value);
		}

		public function get value() : Number
		{
			return _value;
		}

		/**
		 * Gets / sets the maximum value of this slider.
		 */
		public function set maximum(maximum : Number) : void
		{
			_maximum = maximum;
			positionThumb();
		}

		public function get maximum() : Number
		{
			return _maximum;
		}

		/**
		 * Gets / sets the minimum value of this slider.
		 */
		public function set minimum(minimum : Number) : void
		{
			_minimum = minimum;
			positionThumb();
		}

		public function get minimum() : Number
		{
			return _minimum;
		}

		/**
		 * Gets / sets the snap interval of this slider.
		 */
		public function get snapInterval() : Number
		{
			return _snapInterval;
		}

		public function set snapInterval(snapInterval : Number) : void
		{
			_snapInterval = snapInterval;
		}

		/**
		 * Creates and adds the child display objects of this component.
		 */
		override protected function addChildren() : void
		{
			_track.addEventListener(MouseEvent.MOUSE_DOWN, handleTrackClick);
			
			_thumb.addEventListener(MouseEvent.MOUSE_DOWN, handleThumbMouseDown);
			_thumb.buttonMode = true;
			_thumb.useHandCursor = true;
		}

		/**
		 * 
		 */
		protected function calculateValue(position : Number, interactionType : String, clickTarget : String, keyCode : int = undefined) : void
		{
			if(_orientation == Slider.HORIZONTAL) 
			{
				var range : Number = _track.width - _thumb.width;
				position = (position * _track.scaleX) - (_thumb.width * 0.5);
			} 
			else 
			{
				range = _track.height - _thumb.height;
				position = (position * _track.scaleY) - (_thumb.height * 0.5);
			}
			
			var newValue : Number = (position / range) * (maximum - minimum);
			newValue = (minimum + newValue);
			
			doSetValue(newValue, interactionType, clickTarget, keyCode);
		}

		/**
		 * 
		 */
		protected function doSetValue(value : Number, interactionType : String = null, clickTarget : String = null, keyCode : int = undefined) : void
		{
			var oldValue : Number = _value;
			
			if(_snapInterval == 0)
			{
				_value = Math.max(minimum, Math.min(value, maximum));
			}
			else if(_snapInterval == 1)
			{
				_value = Math.max(minimum, Math.min(Math.round(value), maximum));
			}
			else
			{
				var pow : Number = Math.pow(10, getPrecision(snapInterval));
				var snap : Number = _snapInterval * pow;
				var rounded : Number = Math.round(value * pow);
				var snapped : Number = Math.round(rounded / snap) * snap;
				var val : Number = snapped / pow;
				_value = Math.max(minimum, Math.min(val, maximum));
			}
			
			/** Only dispatch if value has changed, dispatch when dragging */
			//if (oldValue != _value && ((liveDragging && clickTarget != null) || (interactionType == InteractionInputType.KEYBOARD))) {
			if (oldValue != _value && clickTarget != null)
			{
				dispatchEvent(new Event(Event.CHANGE));
			}
			
			positionThumb();
		}

		/**
		 * Adjusts position of thumb when value, maximum or minimum have changed.
		 * TODO: Should also be called when slider is resized.
		 */
		protected function positionThumb() : void
		{
			var range : Number;
			
			if(_orientation == Slider.HORIZONTAL)
			{
				range = _track.width - _thumb.width;
				_thumb.x = (_value - _minimum) / (_maximum - _minimum) * range;
			}
			else
			{
				range = _track.height - _thumb.height;
				_thumb.y = (_track.height - _thumb.height) - (_value - _minimum) / (_maximum - _minimum) * range;
			}
		}

		/**
		 * Get the number of decimals of a number. Used by set value().
		 */
		protected function getPrecision(num : Number) : Number
		{
			var s : String = num.toString();
			
			if (s.indexOf(".") == -1) return 0;
			
			return s.split(".").pop().length;
		}

		/**
		 * Handler called when user clicks the background of the slider, causing the handle to move to that point. Only active if backClick is true.
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function handleTrackClick(event : MouseEvent) : void
		{
			var position : Number = (_orientation == Slider.HORIZONTAL) ? _track.mouseX : _track.mouseY;
			calculateValue(position, InteractionInputType.MOUSE, Slider.CLICK_TARGET_TRACK);
			handleThumbMouseDown(null);
		}

		/**
		 * Internal mouseDown handler. Starts dragging the handle.
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function handleThumbMouseDown(event : MouseEvent) : void
		{
			_track.stage.addEventListener(MouseEvent.MOUSE_MOVE, handleStageMouseMove);
			_track.stage.addEventListener(MouseEvent.MOUSE_UP, handleStageMouseUp);
			_track.stage.addEventListener(Event.MOUSE_LEAVE, handleStageMouseUp);
		}

		/**
		 * Internal mouseUp handler. Stops dragging the handle.
		 * @param event : The (Mouse)Event passed by the system.
		 */
		protected function handleStageMouseUp(event : Event) : void
		{
			_track.stage.removeEventListener(MouseEvent.MOUSE_UP, handleStageMouseUp);
			_track.stage.removeEventListener(MouseEvent.MOUSE_MOVE, handleStageMouseMove);
			_track.stage.removeEventListener(Event.MOUSE_LEAVE, handleStageMouseUp);
			
			_thumb.stopDrag();
		}

		/**
		 * Internal mouseMove handler for when the handle is being moved.
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function handleStageMouseMove(event : MouseEvent) : void
		{
			var position : Number = (_orientation == Slider.HORIZONTAL) ? _track.mouseX : _track.mouseY;
			calculateValue(position, InteractionInputType.MOUSE, Slider.CLICK_TARGET_THUMB);
		}
	}
}
